/*  bulgroz.cpp - enjoy on #nasgaia irc channel ;-)
 *
 *  Copyright (c) 2003 by Richard GILL <richard@nasgaia.org>.
 *           
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *           
 *  This program is distributed in the hope that it will be useful, but 
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 *  General Public License for more details.
 *           
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 */

#include <cstdlib>
#include <iostream>
#include <string>
#include <vector>
#include <algorithm>
#include <functional>
using namespace std;
#ifdef WIN32
#include <windows.h>
#endif

const string::value_type g_delimiters[] = " \n\t\r',.;?!=<>-+/*";
const char g_version[] = "0.1.4";
const char g_forbidden[] = "richard";

// translate()
// do the translation on one string, which can be a word or a set of words
// separated by delimiters defined above.
// the parameter passed is directly modified, so this function can be called
// as an algorithm in for_each()
void translate(string& s);

// usage()
// show how to call this program on standard output, and exit is the exit
// argument (do_exit) is different from -1, exit with the do_exit value
// otherwise.
void usage(const char *progname, int do_exit = 0);

// version()
// show the version and Copyright on standard output and exit
void version();

// why()
// see in the definition of the function (you've to understand the french
// language)
void why();

// main program
int main(int argc, char **argv)
{
    // fill the vector of words from cmd line
    vector<string> strs;
    for (int i = 1; i < argc; i ++)
        strs.push_back(argv[i]);

    // analyse the command line to see if there're special args
    if (strs.empty())
        usage(argv[0], 1);

    if (find(strs.begin(), strs.end(), "--help") != strs.end())
        usage(argv[0]);
    else if (find(strs.begin(), strs.end(), "--version") != strs.end())
        version();

    // do the translation on each word
    for_each(strs.begin(), strs.end(), translate);

    // output the result
    for (vector<string>::iterator i = strs.begin(); i != strs.end(); i ++)
    {
#ifdef WIN32
        // on Windows, this program is called from a Dos box, so output
        // characters are in OEM coding, not Ansi, we've to switch to the
        // right Dos coding
        char *buffer = new char[i->length() + 1];
        CharToOem(i->c_str(), buffer);
        cout << buffer << " ";
        delete[] buffer;
#else
        cout << *i << " ";
#endif
    }
    cout << endl;

    return 0;
}

void translate(string& s)
{
    string n;
    string::size_type pos;

    // check if the forbidden word isn't there
    if (s.find(g_forbidden) != string::npos)
        why();

    // try to find special characters which make a word in fact
    // two words for translation
    if ((pos = s.find_first_of(g_delimiters)) != string::npos)
    {
        // split in two words, a make an recursive call
        string s1, s2;
        string::value_type c;
        c = s.at(pos);          // the delimiter
        s1 = s.substr(0, pos);  // before delimiter
        s2 = s.substr(pos + 1); // after delimiter
        translate(s1);          // recursive call for left part
        translate(s2);          //      ""        ""  right  ""
        n = s1;                 // put the translated left part
        n.push_back(c);         // put the delimiter
        n.append(s2);           // put the translated right part
        s = n;                  // replace the original
    }
    else
    {
        // do the translation
        reverse(s.begin(), s.end());
    }
}

void usage(const char *progname, int do_exit)
{
    cout << endl;
    cout << "usage: " << progname;
    cout << " [args] <list of words to translate>" << endl;
    cout << "  with args which are optionnals and can be one of the following:";
    cout << endl;
    cout << "  --help    show this help screen and exit" << endl;
    cout << "  --version show the program version and exit" << endl;
    cout << endl;

    // if exit asked (not -1), exit the program with the value passed
    if (do_exit != -1)
        exit(do_exit);
}

void version()
{
    cout << endl;
    cout << "bulgroz version " << g_version << endl;
    cout << "Copyright (c) 2003 Richard GILL <richard@nasgaia.org>" << endl;
    cout << " \\o/ enjoy on #nasgaia irc channel \\o/" << endl;
    cout << endl;
    exit(0);
}

void why()
{
    cout << endl;
    cout << "Exceptionnellement, j'crirai ceci en franais, car le message ";
    cout << endl;
    cout << "que j'ai besoin de faire passer est de la plus haute importance, ";
    cout << endl;
    cout << "et je ne voudrais pas qu'il y ait confusion sur mes propos.";
    cout << endl;
    cout << endl;
    cout << "Bulgroz a t ncessaire pour sauver le monde, et le restera ";
    cout << endl;
    cout << "pour prserver la paix. Mais comme il s'agit d'un pas logique ";
    cout << endl;
    cout << "ciel, il *doit* tre bourr de bugs (sinon je serais forc d'tre ";
    cout << endl;
    cout << "l'exception qui confirme la rgle); non pas que a m'enchante, ";
    cout << endl;
    cout << "parce que c'est difficile de coder des bugs quand on n'en a pas ";
    cout << endl;
    cout << "l'habitude, mais force est de constater que c'est ncessaire pour ";
    cout << endl;
    cout << "permettre aux lecteurs de linuxfr.org (DLFP pour les intimes) de ";
    cout << endl;
    cout << "s'exprimer avec allgresse.";
    cout << endl << endl;
    cout << "Malheureusement, vous avez appel ce programme avec un mot ";
    cout << endl;
    cout << "interdit, et je suis dans l'obligation de vous rappeler que ";
    cout << endl << endl;
    cout << "   R I C H A R D    N ' E S T    P A S    M E C H A N T ! ! !";
    cout << endl << endl;
    cout << "Ou si vous prfrez:" << endl << endl;
    cout << "   D R A H C I R    N ' T S E    S A P    T N A H C E C ! ! !";
    cout << endl << endl;
    cout << "Voil, j'espre ne pas vous avoir trop drang ;-)." << endl;
    cout << endl;
    exit(0);
}

